#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <err.h>

#include "frontend_getsetpoint.h"
#include "setpoint_common.h"
#include "cli_getsetpoint.h"

/* Get a specified setpoint.
 *
 * Example:
 *
 * struct frontend_setpoint_data setpoint_data;
 * if (frontend_getpid(conn, &pid_data)) {
 * 		error
 * } else {
 * 		setpoint_data.height, setpoint_data.lat, and setpoint.long are filled
 * }
 *
 * Returns 0 on success, 1 on error
 */
int frontend_getsetpoint(
		struct backend_conn * conn, struct frontend_setpoint_data * setpoint_data, int type) {
	
	char line[25] = "";
	switch (type) {
		case HEIGHT :
			strncpy(line, "getheight\n", 10);
			break;
		case LAT :
			strncpy(line, "getlat\n", 7);
			break;
		case LONGG :
			strncpy(line, "getlong\n", 8);
			break;
		case PITCH :
			strncpy(line, "getpitch\n", 9);
			break;
		case ROLL :
			strncpy(line, "getroll\n", 8);
			break;
		case YAW :
			strncpy(line, "getyaw\n", 7);
			break;
	}
	int size;
	if((size = ucart_backendWrite(conn, line)) < 0 ) {
		return 1;
	}

	char * response;
	char tmpresponse[64];
	char * cmdString;
	size_t pendingResponses = 1;
	float value;
	while (pendingResponses) {
		response = ucart_backendGetline(conn);
		if (response == NULL) {
			warnx("Line not returned from backend");
			return 1;
		}
		strncpy(tmpresponse, response, 64);
		if (strncmp(tmpresponse, "get", 3) == 0) {
			cmdString = strtok(tmpresponse, " ");
			value = strtof(&response[strlen(cmdString)],NULL);
			
			switch (type) {
				case HEIGHT :
					setpoint_data->height = value;
					break;
				case LAT :
					setpoint_data->lat = value;
					break;
				case LONGG :
					setpoint_data->longg = value;
					break;
				case PITCH :
					setpoint_data->pitch = value;
					break;
				case ROLL :
					setpoint_data->roll = value;
					break;
				case YAW :
					setpoint_data->yaw = value;
					break;
			}

			pendingResponses--;
		}
	}

	return 0;
}	
